#include "calculator.h"
#include <qlineedit.h>
#include <qtextedit.h>


/*! \mainpage Calculator FSME tutorial
 *
 * @section intro Intro
 *
 * This example introduces Calculator - a sample
 * calculator implementation, when primary logic is
 * drawn.
 *
 * Calculator supports only four types of actions with
 * integer numbers, so it's quite unuseful. I
 * wrote it only to demonstrate logic simplify
 * with usage of finite state machine drawn
 * with <a href="http://fsme.sf.net">FSME</a>.
 *
 * @section hiw How it works
 * The main class is CalculatorFSM, which is drawn with FSME
 * (http://fsme.sf.net) and translated to source code with FSMC.
 *
 * \image html fsm.png "CalculatorFSM diagram"
 *
 * Every button click is an event to CalculatorFSM. It accepts event
 * and processes it according to it's current state. The state is
 * one of CalculatorFSM::States. Look at the outputs of CalculatorFSM
 * and their implementation (for example,  Calculator::saveOp).
 *
 * To be general, all calculator operations are derived from
 * Calculator::Operation. In principle, if there were no good
 * calculator implementations, one could implement more operations
 * to make it a bit more useful :-)
 */





/* 
 *  Constructs a Calculator which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
Calculator::Calculator( QWidget* parent,  const char* name, WFlags fl )
    : CalculatorDlg( parent, name, fl ), enteredOperand( 0 ), enteredOperation( 0 )
{
    eCurrentState->setText( stateName(currentState()) );
    operands.push( 0 );
    showStackTop(); // display initial 0

    eLog->setTextFormat( LogText );
    eLog->setMaxLogLines( 1000 );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Calculator::~Calculator()
{
    // no need to delete child widgets, Qt does it all for us
}

void Calculator::showStackTop()
{
    eResult->setText( QString::number(operands.top()) );
}

/*
 * public slot
 */
void Calculator::slotNumber(int n)
{
    eLog->append( tr("Calculator::slotNumber(<b>%1</b>)").arg(n) );
    enteredOperand = n;
    A( Number );
    eCurrentState->setText( stateName(currentState()) );
}

/*
 * public slot
 */
void Calculator::slotPlus()
{
    eLog->append( tr("Calculator::slotPlus()") );
    if( enteredOperation )
        delete enteredOperation;
    enteredOperation = new OpPlus( this );
    A( BinaryOp );
    eCurrentState->setText( stateName(currentState()) );
}

/*
 * public slot
 */
void Calculator::slotMinus()
{
    eLog->append( tr( "Calculator::slotMinus()" ) );
    if( enteredOperation )
        delete enteredOperation;
    enteredOperation = new OpMinus( this );
    A( BinaryOp );
    eCurrentState->setText( stateName(currentState()) );
}

/*
 * public slot
 */
void Calculator::slotMul()
{
    eLog->append( tr( "Calculator::slotMul()" ) );
    if( enteredOperation )
        delete enteredOperation;
    enteredOperation = new OpMul( this );
    A( BinaryOp );
    eCurrentState->setText( stateName(currentState()) );
}

/*
 * public slot
 */
void Calculator::slotDiv()
{
    eLog->append( tr( "Calculator::slotDiv()" ) );
    if( enteredOperation )
        delete enteredOperation;
    enteredOperation = new OpDiv( this );
    A( BinaryOp );
    eCurrentState->setText( stateName(currentState()) );
}

/*
 * public slot
 */
void Calculator::slotClear()
{
    eLog->append( tr( "Calculator::slotClear()" ) );
    A( Clear );
    eCurrentState->setText( stateName(currentState()) );
}

/*
 * public slot
 */
void Calculator::slotEqual()
{
    eLog->append( tr( "Calculator::slotEqual()" ) );
    A( ResultRequested );
    eCurrentState->setText( stateName(currentState()) );
}


void Calculator::saveNumber()
{
    operands.push( 0 );
}

void Calculator::addNumber()
{
    operands.top() = operands.top() * 10 + enteredOperand;
    showStackTop();
}

void Calculator::clear()
{
    operands.clear();
    operands.push( 0 );
    operations.clear();
    showStackTop();
}

void Calculator::saveOp()
{
    operations.push( enteredOperation );
    enteredOperation = 0;
}

void Calculator::showResult()
{
    Q_ASSERT( !operations.isEmpty() );
    Operation* op = operations.pop();
    (*op)();
    delete op;
    showStackTop();
}


void Calculator::OpPlus::operator()()
{
    int n2 = calc()->popOperand();
    int n1 = calc()->popOperand();
    calc()->pushOperand( n1 + n2 );
}
void Calculator::OpMinus::operator()()
{
    int n2 = calc()->popOperand();
    int n1 = calc()->popOperand();
    calc()->pushOperand( n1 - n2 );
}
void Calculator::OpMul::operator()()
{
    int n2 = calc()->popOperand();
    int n1 = calc()->popOperand();
    calc()->pushOperand( n1 * n2 );
}
void Calculator::OpDiv::operator()()
{
    int n2 = calc()->popOperand();
    int n1 = calc()->popOperand();
    calc()->pushOperand( n1 / n2 );
}
