#ifndef CALCULATOR_H
#define CALCULATOR_H
#include "calc_win.h"
#include "calc.h"

#include <qvaluestack.h>
#include <qptrstack.h>


/** \brief A Sample Calculator class
 *
 * It supports only four types of actions with
 * integer numbers, so it's quite unuseful. I
 * wrote it only to demonstrate logic simplify
 * with usage of finite state machine drawn
 * with FSME.
 */
class Calculator : public CalculatorDlg,
                   public CalculatorFSM
{
    Q_OBJECT

    /** \brief An abstract operation, which acts on Calculator
     * operand stack
     *
     * Calculator has 2 stacks: Calculator::operands
     * and Calculator::operations. The first one holds
     * integer operand values, and the second one holds
     * operations (either unary or binary, but in this demo
     * I implemented only 4 simplest binary operations).
     */
    class Operation {
        Calculator* c;
    protected:
        Calculator* calc() const { return c; }
    public:
        /** A public constructor.
         *
         * @param calc - a parent. It will be used in operator()()
         */
        Operation( Calculator* calc ) : c(calc) {}

        /// Pure virtual operator - it's an operation itself
        /** The operation code is pretty simple. Just pop
         *  up needed number of arguments using Calculator::popOperand(),
         *  perform calculation and put result back via Calculator::pushOperand( int ).
         */
        virtual void operator()() = 0;        
    };

    /// Addition operation
    class OpPlus : public Operation {
    public:
        OpPlus( Calculator* calc ) : Operation( calc ) {}
        virtual void operator()();
    };

    /// Subtraction operation
    class OpMinus : public Operation {
    public:
        OpMinus( Calculator* calc ) : Operation( calc ) {}
        virtual void operator()();
    };

    /// Multiplication operation
    class OpMul : public Operation {
    public:
        OpMul( Calculator* calc ) : Operation( calc ) {}
        virtual void operator()();
    };

    /// Division operation
    class OpDiv : public Operation {
    public:
        OpDiv( Calculator* calc ) : Operation( calc ) {}
        virtual void operator()();
    };

    /** \brief Operands stack.
     *
     * There are 2 stacks: operands and operations. This one holds
     * operands being entered. In our simple case this stack will never
     * have more than 1 element. But if parentheses are introduced,
     * deep will grow.
     * @see Calculator::Operation
     */
    QValueStack<int>     operands;
    /** \brief Operations stack. @see Calculator::Operation
     *
     * There are 2 stacks: operands and operations. This one holds
     * operations being entered. In our simple case this stack will never
     * have more than 1 element. But if parentheses are introduced,
     * deep will grow.
     *
     * After '=' is pressed or ')' is entered, operation will be popped
     * from this stack. Then it will be executed (taking appropriate number
     * of arguments from operands stack) and the result will be pushed to
     * operands stack back.
     * 
     * @see Calculator::Operation
     */
    QPtrStack<Operation> operations;

    /// Entered digit
    int          enteredOperand;
    /** \brief Entered operation
     *
     * This is an operation entered by user. For example,
     * when user clicks "1+2", first, enteredOperand will be "1", and
     * enteredOperation will be Calculator::OpPlus
     */
    Operation*   enteredOperation;

    /// Display a top of stack on a screen. This is our result
    void showStackTop();

public:
    Calculator( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~Calculator();

    /// Pop an operand from operands stack. It's used by Calculator::Operation
    int popOperand()          { return operands.pop(); }
    /// Push result to the operands stack. It's used by Calculator::Operation::operator()()
    void pushOperand( int n ) { operands.push( n );    }

public slots:
    /// A number have veen entered
    void slotNumber(int);
    /// '+' have been pressed
    void slotPlus();
    /// '-' have been pressed
    void slotMinus();
    /// '*' have been pressed
    void slotMul();
    /// '/' have been pressed
    void slotDiv();
    /// 'C' have been pressed
    void slotClear();
    /// '=' have been pressed
    void slotEqual();
protected:
/** \addtogroup from_fsm Inherited FSM outputs.
 *
 * These outputs are implementation of ones from FSM
 *
 *@{*/
    virtual void saveNumber(); 
    virtual void addNumber(); 
    virtual void clear(); 
    virtual void saveOp(); 
    virtual void showResult();
/**@}*/
};

#endif // CALCULATOR_H
